lm.component('fileCompFolderSelect', {
	templateUrl: 'comp/file/fileCompFolderSelect.' + lmCacheTag + 'html',
	bindings: {
		lmFolder: '<',
		selectedFolder: '<',
		onFolder: '&',
		initFolder: '<'
	},
	controller: function($scope, $element, $animate, $http, serviceLimboMedia) {
		var ctrl = this;
		ctrl.states = ['init','load','done']
		ctrl.state = 'init';
		ctrl.open = false;
		
		ctrl.folders = [];
		
		// Init (select/open (sub)folder) facilities:
		ctrl.initFolderPass = null;
		ctrl.autoOpen = false;

		// Init variant 1:
		// There's a folder id to init given but we dont know where it's at,
		// so we load the stack to pass "open-chain" to subfolders.
		ctrl.initLoadStack = function(theInitFolderId) {
			$http.get(lmBase.url + 'f/folder/' + theInitFolderId + '/stack').then(
				function success(resp) {
					ctrl.initFolderPass = resp.data;
					ctrl.initSelf();
				},
				function error(err) {
					ctrl.initSelf();
					serviceLimboMedia.handleHttpErrorNew(err);	
				}
			);			
		};
		
		// Init variant 2:
		// There's a stack given that we need to interpret.
		// If we're first in the stack -> remove self, load subs, pass stack
		// If we're the only in stack -> select 
		ctrl.initReadStack = function(theInitFolderStack) {
			if(theInitFolderStack.length > 0 && theInitFolderStack[0].id == ctrl.lmFolder.id) {
				theInitFolderStack.shift();
				ctrl.autoOpen = true;
				if(theInitFolderStack.length == 0) {
					ctrl.onFolder({folder: ctrl.lmFolder});
					return;
				}
				else {
					ctrl.initFolderPass = theInitFolderStack;
				}
			}
			ctrl.initSelf();
		};
		
		// Init self meaning load own subfolders.
		ctrl.initSelf = function() {
			if(ctrl.lmFolder) {
				if(ctrl.autoOpen) {
					ctrl.open = true;
					ctrl.loadSubfolders(ctrl.lmFolder.id);
				}
				else {
					// wait for click to open
				}
			}
			else {
				// We is root. Init/Open directly.
				ctrl.open = true;
				ctrl.loadSubfolders(0);
			}
		};
		
		ctrl.loadSubfolders = function(folderId) {
			ctrl.state = 'load';
			$http.get(lmBase.url + 'f/folder/' + folderId + '/list').then(
				function success(resp) {
					ctrl.folders = resp.data;
					ctrl.state = 'done';
				},
				function error(err) {
					ctrl.state = 'done';
					serviceLimboMedia.handleHttpErrorNew(err);
				}
			);
		};		

		ctrl.onClick = function() {
			if(ctrl.open) {
				ctrl.open = false;
			}
			else {
				ctrl.open = true;
				if(ctrl.state == 'init') {
					ctrl.loadSubfolders(ctrl.lmFolder.id);
				}
			}
			
			ctrl.onFolder({folder: ctrl.lmFolder});
		};
		
		ctrl.onChildFolder = function(childFolder) {
			// pass upwards
			ctrl.onFolder({folder: childFolder});
		};
		
		/* ======================= */
		/* = LIFECYCLE CALLBACKS = */
		/* ======================= */
				
		ctrl.$onInit = function() {
			if(ctrl.initFolder) {
				if(typeof ctrl.initFolder == 'number') {
					ctrl.initLoadStack(ctrl.initFolder);
				}
				else {
					ctrl.initReadStack(ctrl.initFolder);
				}
			}
			else {
				ctrl.initSelf();
			}
		}
	}
});
