lm.component('fileCompPathSelect', {
	templateUrl: 'comp/file/fileCompPathSelect.' + lmCacheTag + 'html',
	bindings: {
		lmPath: '<',			// The item (BrowsePath) to be shown. null means root and wont render iteself but the children.
		lmSelected: '<',		// The selected item (BrowsePath) passed downwards in tree to be marked where it matches lmPath.
		onSelect: '&',			// Callback for item clicks (with BrowsePath as browsePath). It's up to the surrounding component to use as lmSelected or do something else.  
		initPath: '<'			// Path (String) to open (and select) initially. Will lead to onSelect callback if found.
	},
	controller: function($scope, $element, $animate, $http, serviceLimboMedia) {
		var ctrl = this;
		ctrl.states = ['init','load','done'];
		ctrl.state = 'init';
		ctrl.open = false;
		ctrl.children = [];
		ctrl.initPathPass = null;

		// Init self meaning load own subfolders.
		ctrl.initSelf = function() {
			if(!ctrl.lmPath) {
				// We is root -> Pass init path, open and load children.
				ctrl.initPathPass = ctrl.initPath ? ctrl.initPath : null;
				ctrl.open = true;
				ctrl.loadChildren('');
			}
			else if(ctrl.initPath) {
				// We is sub with init path -> Select, open, pass init path or do nothing depending on how init path is matched.
				var thisPath = ctrl.lmPath.path;
				var initPath = ctrl.initPath;
				var eq = initPath == thisPath;
				var sw = initPath.startsWith(thisPath + (thisPath == '/' ? '' : '/'));
				if(eq) {
					// Exact match means this is the folder we're seaching.
					// Select and stop passing downwards.
					ctrl.onSelect({browsePath: ctrl.lmPath});
				}
				else if(sw) {
					// Start match means this is a parent folder of the searched on.
					// Pass path downwards.
					ctrl.initPathPass = initPath;
					ctrl.open = true;
					ctrl.loadChildren(ctrl.lmPath.path);
				}
			}
			else {
				// We is sub without init -> Do nothing and Wait for user click.
			}
		};
		
		ctrl.loadChildren = function(valPathString) {
			ctrl.state = 'load';
			$http.get(lmBase.url + 'system/browsePath',{params : {path:valPathString}}).then(
				function success(resp) {
					ctrl.children = resp.data;
					ctrl.state = 'done';
				},
				function error(err) {
					ctrl.state = 'done';
					serviceLimboMedia.handleHttpErrorNew(err);
				}
			);
		};

		ctrl.onClick = function() {
			if(ctrl.open) {
				ctrl.open = false;
			}
			else {
				ctrl.open = true;
				if(ctrl.state == 'init') {
					ctrl.loadChildren(ctrl.lmPath.path);
				}
			}
			
			ctrl.onSelect({browsePath: ctrl.lmPath});
		};
		
		ctrl.onChildSelected = function(browsePath) {
			// pass upwards
			ctrl.onSelect({'browsePath': browsePath});
		};
		
		/* ======================= */
		/* = LIFECYCLE CALLBACKS = */
		/* ======================= */
				
		ctrl.$onInit = function() {
			ctrl.initSelf();
		}
	}
});
