/*
 * Params:
 * user: owner
 * id: folder id
 */
lm.component('fileDialogFolderShare', {
	templateUrl: 'comp/file/fileDialogFolderShare.' + lmCacheTag + 'html',
	bindings: { lmParams: '<', lmDismiss: '&', lmDone: '&' },
	controller: function($http, serviceLimboMedia, serviceNotification) {
		var ctrl = this;
		
		ctrl.earl = lmBase.urlFull + '#/link/';
		ctrl.availableRights = ['READ','WRITE'];
		ctrl.availableUsers = [];
		ctrl.availableRanges = [{name: 'DAY',val: 86400000},{name:'WEEK',val: 604800000},{name: 'MONTH',val: 2592000000},{name:'YEAR',val: 31536000000}];
		ctrl.userNew = {right: ctrl.availableRights[0], user: null};
		ctrl.tokenNew = {name: '', right: ctrl.availableRights[0], range: ctrl.availableRanges[1]};
		ctrl.share = null;

		ctrl.$onInit = function() {
			if(!ctrl.lmParams.user) {
				console.log("No user param. Closing.");
				ctrl.lmDismiss();
				return;
			}
			if(!ctrl.lmParams.id) {
				console.log("No id param. Closing.");
				ctrl.lmDismiss();
				return;
			}

			/* load available users */
			$http.get(lmBase.url + 'user/users').then(
				function success(resp) {
					ctrl.availableUsers = resp.data;
					ctrl.userNew.user = ctrl.availableUsers[0];
				},
				serviceLimboMedia.handleHttpErrorNew
			);

			/* load shares */
			$http.get(lmBase.url + 'f/folder/' + ctrl.lmParams.id + '/share').then(
				function success(resp) {
					ctrl.share = resp.data;
				},
				serviceLimboMedia.handleHttpErrorNew
			);		
		};
		
		ctrl.userAdd = function() {
			if(ctrl.lmParams.user.id == ctrl.userNew.user.id) {
				serviceNotification.error('MSG_SHARE_YOURSELF');
				return;
			}
			
			for(x in ctrl.share.users) {
				if(ctrl.userNew.user.id == ctrl.share.users[x].user.id) {
					serviceNotification.error('MSG_SHARE_TWICE');
					return;
				}
			}
			
			var uwr = {user: ctrl.userNew.user, right: ctrl.userNew.right};
			ctrl.share.users.push(uwr);
		};
		
		ctrl.userRemove = function(valUser) {
			lmBase.arrayRemove(ctrl.share.users, valUser);
		};
		
		ctrl.tokenAdd = function() {
			$http.get(lmBase.url + 'util/uid').then(
				function success(resp) {
					var newToken = {
						id: resp.data,
						name: ctrl.tokenNew.name,
						right: ctrl.tokenNew.right,
						expires: Date.now() + ctrl.tokenNew.range.val,
						resourceType: 'FOLDER',
						resourceId: ctrl.lmParams.id
					};
					ctrl.share.tokens.push(newToken);
				},
				serviceLimboMedia.handleHttpErrorNew
			);
		};
		ctrl.tokenRemove = function(valueToken) {
			lmBase.arrayRemove(ctrl.share.tokens, valueToken);
		};
		
		ctrl.tokenLink = function(valToken) {
			return ctrl.earl + valToken.id + '/' + valToken.resourceType + '/' + valToken.resourceId;
		};
		
		ctrl.ok = function() {
			$http.post(lmBase.url + 'f/folder/' + ctrl.lmParams.id + '/share', ctrl.share).then(
				function success(resp) {
					ctrl.lmDone();
				},
				serviceLimboMedia.handleHttpErrorNew
			);
		};
		
	}
});
