/*
 * Params:
 * folderTarget: (Current) folder where to potentially move/copy files to.
 * action: 'copy','move','delete' to trigger action or null to just show.
 * Output:
 * success -> never
 * reject  -> with true on potential changes (for refresh...), false on no change
 */
lm.component('fileDialogSelection', {
	templateUrl: 'comp/file/fileDialogSelection.' + lmCacheTag + 'html',
	bindings: { lmParams: '<', lmState: '=', lmDismiss: '&', lmDone: '&' },
	controller: function($element, $http, serviceLimboMedia, serviceNotification, serviceFileSelect) {
		var ctrl = this;
		
		ctrl.state = serviceFileSelect.state;
		
		ctrl.folderTarget = null;
		ctrl.items = [];
		
		ctrl.$onInit = function() {
			if(ctrl.lmParams.folderTarget) {
				ctrl.folderTarget = ctrl.lmParams.folderTarget;
			}
			ctrl.refresh();
			if(ctrl.lmParams.action) {
				ctrl.doAction(ctrl.lmParams.action);
			}
		};

		ctrl.refresh = function() {
			ctrl.items = Array.from(serviceFileSelect.valuesWrapped());
		};
		
		ctrl.itemRemove = function(valItem) {
			serviceFileSelect.remove(valItem);
			ctrl.refresh();
			if(ctrl.items.length == 0) ctrl.lmDismiss();
		};

		ctrl.callbackFilesChanged = {
			onRemove: function(itemRemoved) {
				// file has been moved/copied -> removed
				ctrl.refresh();
			},
			onChange: function(itemOriginal, itemChanged) {
				// file.error has been added -> refresh
				ctrl.refresh();
			}
		};
		
		ctrl.doAction = function(action) {
			ctrl.lmState.closable = false;
			ctrl.lmState.closeResult = true;
			serviceFileSelect.action(action, ctrl.folderTarget, ctrl.callbackFilesChanged).then(
				ok => {
					ctrl.lmState.closable = true;
					ctrl.lmDismiss();
				},
				err => {
					ctrl.lmState.closable = true;
				}
			);
		};
		
		ctrl.doClear = function() {
			serviceFileSelect.clear();
			ctrl.lmDismiss();
		};

	}
});
