lm.component('filePage', {
  templateUrl: 'comp/file/filePage.' + lmCacheTag + 'html',
  bindings: {
  },
  $canActivate: function($nextInstruction, $prevInstruction, serviceLimboMedia) {
	  return serviceLimboMedia.isAuthenticated();
  },
  controller: function($scope, $http, $timeout, $rootRouter, serviceFile, serviceImager, serviceNotification, serviceLimboMedia, serviceMediaPlayer, serviceUpload, serviceFileSelect, serviceModal, serviceGlobalEvents, serviceContext) {
		var ctrl = this;
	  	ctrl.serviceFile = serviceFile;
		ctrl.serviceImager = serviceImager;
		
		/* ========== */
		/* = SEARCH = */
		/* ========== */	
		ctrl.search = {active: false, value: ''};
		ctrl.searchClear = function() {
			ctrl.search.active = false;
			ctrl.search.value = '';
		};
		ctrl.onSearchChanged = function() {
			ctrl.folderLoad();
		};

		/* ========== */
		/* = FILTER = */
		/* ========== */
		ctrl.filters = [
			{id: 'ALL',   i18n: 'FFILTER_NONE',  icon: 'lmi-file'		, order: 'NAME'},
			{id: 'IMAGE', i18n: 'FFILTER_IMAGE', icon: 'lmi-file-image', order: 'NAME'},
			{id: 'AUDIO', i18n: 'FFILTER_AUDIO', icon: 'lmi-file-audio', order: 'NAME'},
			{id: 'VIDEO', i18n: 'FFILTER_VIDEO', icon: 'lmi-file-video', order: 'NAME'}
		];
		
		ctrl.filter = ctrl.filters[0];
		
		ctrl.toggleFilter = function() {
			ctrl.filter = lmBase.arrayGetNext(ctrl.filters, ctrl.filter);
			ctrl.predicate = ctrl.filter.order;
			ctrl.settings.filter = ctrl.filter.id;
			ctrl.saveSettings();
			ctrl.filesLoad();
		}

		/* ======== */
		/* = VIEW = */
		/* ======== */
		ctrl.views = [
			{id: 'LIST',    i18n: 'LIST',    icon: 'lmi-layout-list'},
			{id: 'GALLERY', i18n: 'GALLERY', icon: 'lmi-layout-grid'}
		];
		
		ctrl.view = ctrl.views[0];

		ctrl.toggleView = function() {
			ctrl.view = lmBase.arrayGetNext(ctrl.views, ctrl.view);
			ctrl.settings.view = ctrl.view.id;
			ctrl.saveSettings();
			ctrl.filesLoad();
		}
		
		ctrl.listClazz = 'vM';
		ctrl.listThumb = 'TXS_QC';
		ctrl.listShowThumb = true;
		
		ctrl.galleryClazz = 'vM';
		ctrl.galleryThumb = 'TL_H';

		ctrl.refreshDesign = function() {
			if('SMALL' == ctrl.settings.list.size) {
				ctrl.listClazz = 'vS';
				ctrl.listThumb = 'NONE';
				ctrl.listShowThumb = false;
			}
			else if('LARGE' == ctrl.settings.list.size) {
				ctrl.listClazz = 'vL';
				ctrl.listThumb = 'TS_QC';
				ctrl.listShowThumb = true;
			}
			else {
				ctrl.listClazz = 'vM';
				ctrl.listThumb = 'TXS_QC';
				ctrl.listShowThumb = true;
			}
	
			if('SMALL' == ctrl.settings.gallery.size) {
				ctrl.galleryClazz = 'vS';
				ctrl.galleryThumb = 'TM_H';
			}
			else if('LARGE' == ctrl.settings.gallery.size) {
				ctrl.galleryClazz = 'vL';
				ctrl.galleryThumb = 'TXL_H';
			}
			else {
				ctrl.galleryClazz = 'vM';
				ctrl.galleryThumb = 'TL_H';
			}
		};
		
		/* ========= */
		/* = ORDER = */
		/* ========= */
		ctrl.predicate = 'NAME';
		ctrl.dir = 'asc';	
		
		ctrl.changeOrder = function(valueOrder) {
			if(ctrl.predicate == valueOrder) {
				ctrl.dir = ctrl.dir == 'asc' ? 'desc' : 'asc';
			}
			else {
				ctrl.dir = 'asc';
				ctrl.predicate = valueOrder;
			}
			ctrl.filesLoad();
		};
		
		/* =============================== */
		/* = CURRENT FOLDER / SUBFOLDERS = */
		/* =============================== */
		
		ctrl.folderId = 0;				/* folder id as given via url */
		
		ctrl.folder = null;				/* current folder */
		ctrl.folderStack = [];			/* folder stack (for breadcrumb) */
		ctrl.folders = [];				/* list of subfolders */
		
		ctrl.folderLoad = function() {
			ctrl.folders = [];

			if(ctrl.search.active) {
				var par = {folder: ctrl.folderId, fr: 0, mr: 100, s: ctrl.search.value};
				$http.get(lmBase.url + 'f/folder/search',{params : par}).then(
					function success(resp) {
						ctrl.folders = resp.data;
						ctrl.filesLoad();
					},
					serviceLimboMedia.handleHttpErrorNew
				);
			}
			else {
				$http.get(lmBase.url + 'f/folder/' + ctrl.folderId + '/stacklist').then(
					function success(resp) {
						ctrl.folderStack = resp.data.stack;
						ctrl.folder = resp.data.stack.length == 0 ? null : resp.data.stack[resp.data.stack.length - 1];
						ctrl.folders = resp.data.items;
						ctrl.filesLoad();
					},
					serviceLimboMedia.handleHttpErrorNew
				);
			}
		};
		
		/* ============== */
		/* = FOLDER OPS = */
		/* ============== */	
		
		ctrl.folderDelete = function(valueItem) {
			/* Old confirm (choose delete/unbind) dialog 
			serviceModal.show('file-dialog-folder-delete',{folder: valueItem}).then(
				function() {
					var idx = ctrl.folders.indexOf(valueItem);
					if(idx != -1) ctrl.folders.splice(idx,1);
				},
				function() {}
			);
			*/

			var prom = $http.delete(lmBase.url + 'f/folder/' + valueItem.id).then(
				function success(resp) {
					var idx = ctrl.folders.indexOf(valueItem);
					if(idx != -1) ctrl.folders.splice(idx,1);
				},
				serviceLimboMedia.handleHttpErrorNew
			);
			serviceNotification.showWorking(prom);
		};
		
		ctrl.folderUnbind = function(valueItem) {
			var prom = $http.post(lmBase.url + 'f/folder/' + valueItem.id + '/unbind').then(
				function success(resp) {
					var idx = ctrl.folders.indexOf(valueItem);
					if(idx != -1) ctrl.folders.splice(idx,1);
				},
				serviceLimboMedia.handleHttpErrorNew
			);
			serviceNotification.showWorking(prom);
		};

		ctrl.folderShare = function(valueItem) {
			serviceModal.show('file-dialog-folder-share',{user: valueItem.x.user, id: valueItem.id}).then(
				  function() { ctrl.folderLoad(); },
				  function() { }
			);
		};
		
		ctrl.folderCreate = function() {
			var par = {parentId: ctrl.folder ? ctrl.folder.id : 0};
			serviceModal.show('file-dialog-folder-edit', par).then(
				function() { ctrl.folderLoad(); },
				function() {}
			);
		};

		ctrl.folderEdit = function(valueItem) {
			var par = {id: valueItem.id};
			serviceModal.show('file-dialog-folder-edit', par).then(
				function(result) { 
					var idx = ctrl.folders.indexOf(valueItem);
					if(result && idx != -1) {
						ctrl.folders[idx] = result;
					}
					// ctrl.folderLoad();
				},
				function() {}
			);
		};	
		
		ctrl.folderSynchronize = function(valueFolder) {
			$http.post(lmBase.url + 'f/folder/' + valueFolder.id + '/sync').then(
				function success(resp) {
					serviceNotification.ok('MSG_JOB_STARTED');
				},
				serviceLimboMedia.handleHttpErrorNew
			);
		};
		
		ctrl.folderMount = function() {
			serviceModal.show('file-dialog-folder-mount').then(
				function() { ctrl.folderLoad(); },
				function() {}
			);
		};
		
		// call $ctrl.fileContext(this, $event) to get right scope even in ng-repeat
		ctrl.folderContext = function(valScope, valEvent) {
			serviceContext.show(valScope ? valScope : $scope, valEvent.target, 'comp/file/filePageContextFolder.' + lmCacheTag + 'html');
		};

		/* ================== */
		/* = FILE SELECTION = */
		/* ================== */
		ctrl.serviceFileSelect = serviceFileSelect;
		
		ctrl.selectionFloaterClick = function(event) {
			serviceContext.show($scope, event.target, 'comp/file/filePageContextSelection.' + lmCacheTag + 'html');
		};
		
		ctrl.selectionDownload = function() {
			var filesInSelection = ctrl.serviceFileSelect.values().filter(fileOrFolder => fileOrFolder.type);
			var filesCsv = filesInSelection.map(file => file.id).join(',');
			var foldersInSelection = ctrl.serviceFileSelect.values().filter(fileOrFolder => !fileOrFolder.type);
			var foldersCsv = foldersInSelection.map(file => file.id).join(',');
			
		    var ifrm = document.getElementById('downloadframe');
			ifrm.src = lmBase.url + 'f/streamZip/LimboDownloadFiles.zip?fileIds=' + filesCsv + '&folderIds=' + foldersCsv;
			ctrl.serviceFileSelect.clear();
		};
		
		ctrl.selectionPlay = function() {
			var relevant = ctrl.serviceFileSelect.values().filter(fileOrFolder => 'AUDIO' == fileOrFolder.type || 'VIDEO' == fileOrFolder.type || 'STREAM' == fileOrFolder.type);
			serviceMediaPlayer.playFiles(relevant);
			ctrl.serviceFileSelect.clear();
		};

		ctrl.selectionAdd = function() {
			var relevant = ctrl.serviceFileSelect.values().filter(fileOrFolder => 'AUDIO' == fileOrFolder.type || 'VIDEO' == fileOrFolder.type || 'STREAM' == fileOrFolder.type);
			serviceMediaPlayer.addFiles(relevant);
			ctrl.serviceFileSelect.clear();
		};

		ctrl.selectionToggleAllFiles = function() {
			if(ctrl.files.length > 0) {
				var firstSelected = ctrl.serviceFileSelect.has(ctrl.files[0]);
				if(firstSelected) ctrl.files.forEach(el => ctrl.serviceFileSelect.remove(el));	
				else ctrl.files.forEach(el => ctrl.serviceFileSelect.add(el));
			}
		};
		
		ctrl.selectionToggleAllFolders = function() {
			if(ctrl.folders.length > 0) {
				var firstSelected = ctrl.serviceFileSelect.has(ctrl.folders[0]);
				if(firstSelected) ctrl.folders.forEach(el => ctrl.serviceFileSelect.remove(el));	
				else ctrl.folders.forEach(el => ctrl.serviceFileSelect.add(el));
			}
		};
		

		ctrl.selectionShow = function(theAction) {
			serviceModal.show('file-dialog-selection', {folderTarget: (ctrl.search.active ? null : ctrl.folder), action: theAction}, {maximize: false}).then(
				function(result) {},
				function(err) {
					if(err) {
						/* things may have been changed, so reload for sure. */
						ctrl.folderLoad();
					}
				}
			);
		};
		
		/* ========= */
		/* = FILES = */
		/* ========= */
		ctrl.files = [];
		
		/* no of items in view */
		ctrl.fileLimit = 0;
		
		ctrl.filesClear = function() {
			ctrl.files = [];
			ctrl.fileLimit = ctrl.settings.pageSize;
		};
		
		ctrl.filesLoad = function() {
			ctrl.filesClear();
			
			var par = {
				order: ctrl.predicate,
				dir: ctrl.dir,
				details: false
			};
			
			if('IMAGE' == ctrl.filter.id) par.fileType = 'IMAGE';
			if('AUDIO' == ctrl.filter.id) par.fileType = 'AUDIO';
			if('VIDEO' == ctrl.filter.id) par.fileType = 'VIDEO';
			
			if(ctrl.search.active) {
				par.mr = 500;
				par.s = ctrl.search.value;
				if(ctrl.folder != null) {
					par.folder = ctrl.folder.id;
				}
				
				$http.get(lmBase.url + 'f/files/search',{params : par}).then(
					function success(resp) {
						for(x in resp.data) {
							var it = resp.data[x];
							lmBase.pimpLmFile(it);
							ctrl.files.push(it);
						}
					},
					serviceLimboMedia.handleHttpErrorNew
				);
			}
			else {
				if(ctrl.folder == null) return;
				
				par.folder = ctrl.folder.id;
				
				$http.get(lmBase.url + 'f/files/list',{params : par}).then(
					function success(resp) {
						for(x in resp.data) {
							var it = resp.data[x];
							lmBase.pimpLmFile(it);
							ctrl.files.push(it);
						}
					},
					serviceLimboMedia.handleHttpErrorNew
				);
			}
		};
		
		ctrl.filesShowMore = function() {
			ctrl.fileLimit += ctrl.settings.pageSize;
		};

		ctrl.filesShowAll = function() {
			ctrl.fileLimit = null;
		};
		
		ctrl.callbackFilesChanged = {
			onRemove: function(itemRemoved) {
				var idx = ctrl.files.indexOf(itemRemoved);
				if(idx != -1) ctrl.files.splice(idx,1);
				serviceFileSelect.remove(itemRemoved);
			},
			onChange: function(itemOriginal, itemChanged) {
				lmBase.pimpLmFile(itemChanged);
				var idx = ctrl.files.indexOf(itemOriginal);
				if(idx != -1) {
					ctrl.files[idx] = itemChanged;
				}
				if(serviceFileSelect.has(itemOriginal)) {
					serviceFileSelect.remove(itemOriginal);
					serviceFileSelect.add(itemChanged);
				}
			}
		};

		ctrl.getSelectedFiles = function(typefilter) {
			var result = new Array();
			
			for(x in ctrl.files) {
				var it = ctrl.files[x];
				if(it.client.select && (typefilter == null || typefilter(it.type))) {
					result.push(it);
				}
			}
			
			return result;
		};

		// call $ctrl.fileContext(this, $event) to get right scope even in ng-repeat
		ctrl.fileContext = function(valScope, valEvent) {
			serviceContext.show(valScope ? valScope : $scope, valEvent.target, 'comp/file/filePageContextFile.' + lmCacheTag + 'html');
		};

		/* ============ */
		/* = FILE OPS = */
		/* ============ */
		
		ctrl.defaultFileAction = function(valueFile) {
			serviceFile.fileDefaultAction(valueFile, ctrl.files, ctrl.folder ? ctrl.folder.x.write : false, ctrl.callbackFilesChanged);
		};
		
		ctrl.mediaPlay = function(valueItem) {
			serviceMediaPlayer.playFile(valueItem);
		};
		
		ctrl.mediaAdd = function(valueItem) {
			serviceMediaPlayer.addFile(valueItem);
		};

		ctrl.mediaAddPlay = function(valueItem) {
			serviceMediaPlayer.addPlayFile(valueItem);
		};
		
		ctrl.mediaExternal = function(valueItem) {
			serviceModal.show('file-dialog-file-links',{id: valueItem.id}).then(function() {}, function() {});
		};

		ctrl.openFolder = function(valueItem) {
			$http.get(lmBase.url + 'f/file/' + valueItem.id + '/folder').then(
				function success(resp) {
					ctrl.searchClear();
					$rootRouter.navigate(['FilesAndMedia', {id: resp.data.id}]);
				}, serviceLimboMedia.handleHttpErrorNew
			);
		};

		/* ============= */
		/* = NEW STUFF = */
		/* ============= */
		ctrl.uploadFiles = [];
		
		/* Functions for direct JS calls (non-angular) */	
		ctrl.onFileSelect = function(valueFiles) {
			$scope.$apply(function() {
				if(!ctrl.folder) {
					serviceNotification.error('MSG_NO_ROOT');
					return;
				}
				
				if(!ctrl.folder.x.write) {
					serviceNotification.error('MSG_NO_RIGHTS');
					return;
				}

				var files = new Array();
			
				for(x in valueFiles) {
					var el = valueFiles[x];
					if(el instanceof File) {
						files.push(el);
					}
				};
				/* let upload service manage this */
				var par = new Object();
				par.folder = ctrl.folder == null ? 0 : ctrl.folder.id;
				serviceUpload.uploadFiles(files,lmBase.url + 'f/file', par);
			});
		};
		
		ctrl.onClassicUpload = function(element) {
			ctrl.onFileSelect(element.files);
			 // Reset input to fire onChange even if user selects same file.
			element.value = "";
		};
		
		ctrl.triggerUpload = function() {
			document.getElementById('fileSelectore').click();
		};
		
		ctrl.addFloaterClick = function(valEvent) {
			if(ctrl.folder) {
				serviceContext.show($scope, valEvent.target, 'comp/file/filePageContextAddSub.' + lmCacheTag + 'html');
			}
			else {
				serviceContext.show($scope, valEvent.target, 'comp/file/filePageContextAddRoot.' + lmCacheTag + 'html');
			}
		};		
		

		/* ====================== */
		/* = DragDrop callbacks = */
		/* ====================== */
		
		ctrl.onDdCheck = function() {
			if(!ctrl.folder) return {allowed: false, msg: 'MSG_NO_ROOT'};
			if(!ctrl.folder.x.write) return {allowed: false, msg: 'MSG_NO_RIGHTS'};
			return {allowed: true, msg: 'MSG_UPLOAD_DROP'};
		};

		ctrl.onDdDrop = function(valueDataTransfer) {
			ctrl.onFileSelect(valueDataTransfer.files);
		};

		/* ======================================= */
		/* = (De)register on events and services = */
		/* ======================================= */
		
		serviceUpload.registerListener(ctrl.folderLoad);
		
		/* Listen to global filedrop */
		serviceGlobalEvents.registerListenerDataTransfer(ctrl);
		
		/* destructor */
		$scope.$on('$destroy', function iVeBeenDismissed() {
			serviceUpload.registerListener(null);
			serviceGlobalEvents.registerListenerDataTransfer(null);
		});
		
		ctrl.fileLinkCreate = function() {
			serviceModal.show('file-dialog-file-link-edit',{folderId: ctrl.folder.id}).then(
				function(result) {
					ctrl.folderLoad();
				},
				function() {}
			);
		};
		ctrl.fileNoteCreate = function() {
			serviceModal.show('file-dialog-file-note-edit',{folderId: ctrl.folder.id}).then(
				function(result) {
					ctrl.folderLoad();
				},
				function() {}
			);
		};
		ctrl.fileTaskCreate = function() {
			serviceModal.show('file-dialog-file-task-edit',{folderId: ctrl.folder.id}).then(
				function(result) {
					ctrl.folderLoad();
				},
				function() {}
			);
		};
		ctrl.fileStreamCreate = function() {
			serviceModal.show('file-dialog-file-stream-edit',{folderId: ctrl.folder.id}).then(
				function(result) {
					ctrl.folderLoad();
				},
				function() {}
			);
		};

		/* ========= */
		/* = IMAGE = */
		/* ========= */
		ctrl.imageShow = function(valueFile) {
			// TODO other writable for searches???
			serviceImager.openImages(valueFile, ctrl.files, ctrl.folder.x.write);
		};
		
		/* ============ */
		/* = SETTINGS = */
		/* ============ */
		ctrl.settings = new Object();
			
		ctrl.toggleConfig = function() {
			serviceModal.show('file-dialog-file-settings').then(
				function() { ctrl.refreshDesign(); ctrl.folderLoad(); },
				function() { ctrl.refreshDesign(); ctrl.folderLoad(); }
			);
		};
		
		ctrl.saveSettings = function() {
			serviceLimboMedia.saveRemoteSettings();
		};	
		
		ctrl.loadSettings = function() {
			ctrl.settings = serviceLimboMedia.getRemoteSettings('files');
			lmBase.defaultSettingsFiles(ctrl.settings);

			ctrl.view   = lmBase.arrayGetByIdOrFirst(ctrl.views, ctrl.settings.view);
			ctrl.filter = lmBase.arrayGetByIdOrFirst(ctrl.filters, ctrl.settings.filter);  
 
			/* =========== */
			/* = STARTUP = */
			/* =========== */
			ctrl.refreshDesign();
			ctrl.folderLoad();
		}
		
		ctrl.$routerOnActivate = function(next) {
		    var id = next.params.id;
		    if(id) {
		    	ctrl.folderId = id;
		    }
		    ctrl.loadSettings();
		};
	  
  }
});