lm.component('lmEditor', {
  templateUrl: 'comp/generic/lmEditor.' + lmCacheTag + 'html',
  bindings: {
	  model: '=',
	  filename: '<',
	  readonly: '<',
	  scrollbottom: '<',
	  titlekey: '@',
	  height: '<',
	  grow: '@'
  },
  controller: function($scope, $element, $http, $translate, serviceLimboMedia, serviceNotification, serviceDeps) {
	  var ctrl = this;
	  
	  ctrl.editorDiv = $element[0].getElementsByClassName('aceditor')[0];
	  ctrl.editorAce = null;
	  
	  ctrl.selfchange = false;
	  
	  ctrl.themes = [
		  {name: 'Monokai', key: 'ace/theme/monokai', optStyle: 'dark'},
		  {name: 'Chrome', key: 'ace/theme/chrome', optStyle: 'bright'},
		  {name: 'Chaos', key: 'ace/theme/chaos', optStyle: 'dark'},
		  {name: 'Gob', key: 'ace/theme/gob', optStyle: 'dark'},
		  {name: 'Idle Fingers', key: 'ace/theme/idle_fingers', optStyle: 'dark'},
		  {name: 'Xcode', key: 'ace/theme/xcode', optStyle: 'bright'}
	  ];
	  ctrl.theme = ctrl.themes[0];
	  
	  ctrl.modes = [
		  {name: 'Text', key: 'ace/mode/text', exts: []},
		  {name: 'C / C++', key: 'ace/mode/c_cpp', exts: ['.c', '.cpp']},
		  {name: 'CSS', key: 'ace/mode/css', exts: ['.css',]},
		  {name: 'HTML', key: 'ace/mode/html', exts: ['.htm', '.html']},
		  {name: 'Ini', key: 'ace/mode/ini', exts: ['.ini',]},
		  {name: 'Java', key: 'ace/mode/java', exts: ['.java',]},
		  {name: 'JavaScript', key: 'ace/mode/javascript', exts: ['.js']},
		  {name: 'Json', key: 'ace/mode/json', exts: ['.json',]},
		  {name: 'Makefile', key: 'ace/mode/makefile', exts: ['makefile',]},
		  {name: 'Markdown', key: 'ace/mode/markdown', exts: ['.md','.MD']},
		  {name: 'PHP', key: 'ace/mode/php', exts: ['.php',]},
		  {name: 'Plain Text', key: 'ace/mode/plain_text', exts: []},
		  {name: 'Properties', key: 'ace/mode/properties', exts: ['.properties',]},
		  {name: 'Python', key: 'ace/mode/python', exts: ['.py',]},
		  {name: 'SH', key: 'ace/mode/sh', exts: ['.sh',]},
		  {name: 'SQL', key: 'ace/mode/sql', exts: ['.sql',]},
		  {name: 'SVG', key: 'ace/mode/svg', exts: ['.svg',]},
		  {name: 'Typescript', key: 'ace/mode/typescript', exts: ['.ts',]},
		  {name: 'XML', key: 'ace/mode/xml', exts: ['.xml',]},
		  {name: 'Yaml', key: 'ace/mode/yaml', exts: ['.yaml',]}
	  ];
	  ctrl.mode = ctrl.modes[0];
	  
	  ctrl.changeTheme = function(valueTheme) {
		  ctrl.theme = valueTheme;
		  if(ctrl.editorAce) {
			  ctrl.editorAce.setTheme(ctrl.theme.key);
		  }
		  ctrl.settingsSave();
	  };
	  
	  ctrl.changeMode = function(valueMode) {
		  ctrl.mode = valueMode;
		  if(ctrl.editorAce) {
			  ctrl.editorAce.getSession().setMode(ctrl.mode.key);
		  }
	  };
	  
	  /* Load settings for last theme */
	  ctrl.settingsLoad = function() {
		  ctrl.settings = serviceLimboMedia.getRemoteSettings('editor');
		  if(ctrl.settings.theme == null) ctrl.settings.theme = 'ace/theme/monokai';
		  
		  var matchingTheme = ctrl.themes.filter(function(mo) {
			  return ctrl.settings.theme == mo.key;
		  })[0];
		  if(matchingTheme) {
			  ctrl.theme = matchingTheme;
		  }
	  }
	  
	  ctrl.settingsSave = function() {
		  ctrl.settings.theme = ctrl.theme.key;
		  serviceLimboMedia.saveRemoteSettings();
	  }
	  
	  ctrl.$onInit = function() {
		  ctrl.settingsLoad();

		  $scope.$watch('$ctrl.model', function(someValue) {
			  ctrl.updateEditor();
		  });
	  };
	  
	  ctrl.$postLink = function() {
		  serviceDeps.load("ace").then(ctrl.initEditor);
	  };
	  
	  ctrl.initEditor = function() {
		  ctrl.editorAce = ace.edit(ctrl.editorDiv);
		  ctrl.editorAce.setShowPrintMargin(false);
		  ctrl.editorAce.$blockScrolling = Infinity;
		  ctrl.editorAce.setOptions({fontFamily: "noto-mono",fontSize: "10pt"});

		  if(ctrl.theme) ctrl.editorAce.setTheme(ctrl.theme.key);
		  if(ctrl.mode) ctrl.editorAce.getSession().setMode(ctrl.mode.key);
		  if(ctrl.readonly) ctrl.editorAce.setReadOnly(ctrl.readonly);
		  
		  ctrl.editorAce.on("change", function(e) {
			  if(!ctrl.selfchange) {
				  $scope.$apply(function() {
					  ctrl.model = ctrl.editorAce.getValue();
				  });
			  }
		  });
		  ctrl.updateEditor();
	  }
	  
	  /* call to update editor from model. Prevent own onchange events. */
	  ctrl.updateEditor = function() {
		  ctrl.selfchange = true;
		  if(ctrl.editorAce && ctrl.editorAce.getValue() != ctrl.model) {
			  if(ctrl.scrollbottom) {
				  ctrl.editorAce.setValue(ctrl.model,1);
				  var rows = ctrl.editorAce.getSession().getLength();
				  ctrl.editorAce.scrollToLine(rows, true, true, function () {});
			  }
			  else {
				  ctrl.editorAce.setValue(ctrl.model,-1);
			  }
		  }
		  ctrl.selfchange = false;
	  };	  
	  
	  ctrl.$onChanges = function(cobj) {
		  if(cobj.filename) {
			  // select mode matching filename
			  if(cobj.filename.currentValue) {
				var matchingmode = ctrl.modes.filter(function(mo) {
					for(iex in mo.exts) {
						if(cobj.filename.currentValue.endsWith(mo.exts[iex])) return true;
					}
					return false;
				})[0];
				if(matchingmode) {
					ctrl.mode = matchingmode;
				}
			  }
			  else {
				  ctrl.mode = ctrl.modes[0];
			  }
			  
			  if(ctrl.editorAce) {
				  ctrl.editorAce.getSession().setMode(ctrl.mode.key);
			  }
		  }
		  if(ctrl.editorAce) {
			  if(cobj.readonly) ctrl.editorAce.setReadOnly(ctrl.readonly);
		  }
	  };
  }
});
