lm.factory('serviceContext', [ '$q', '$compile', '$timeout', '$templateRequest', function($q, $compile, $timeout, $templateRequest) {
	var service = new Object();

	service.instances = new Set();

	// INTERNAL
	service.onClickAnywhere = function(event) {
		/* prevent closing popper if element or parent has class 'noclose'. */
		if(service.instances.size == 0 || event.target.closest('.noclose')) {
			return;
		}
		service.dismiss();
	};

	service.popper = function(instance) {
		// Init in next cycle so click event doesn't close things directly
		$timeout(function() {
			instance.popper = Popper.createPopper(instance.elBinder, instance.elContent, {
				placement: instance.placement,
				modifiers: [{name: 'offset', options: {offset: [0, 8]}}]
			});
			instance.elContent.style.visibility = 'visible';
			service.instances.add(instance);
		});
	};

	// PUBLIC
	service.show = function(valScope, valElBinder, valTemplateUrl, valOptions) {
		valOptions = valOptions || {};
		var instance = {
			templateUrl: valTemplateUrl,
			scope: valScope,										// scope to link to
			elBinder: valElBinder,									// element to place popper around
			elContent: null,										// element with popper content
			elContainer: valOptions.container || document.body,		// Element to put popper in
			popper: null,											// popper.js instance
			placement: valOptions.placement || 'bottom-end' 
		};
		
		// Get external template via caching templateRequest service
		// Compile to angular template -> link to scope -> put in body -> pop
		$templateRequest(instance.templateUrl).then(function(html) {
			var linkFn = $compile(html);
			instance.elContent = linkFn(instance.scope)[0];
			instance.elContainer.appendChild(instance.elContent);
			service.popper(instance);
		});
	};
	
	service.dismiss = function() {
		service.instances.forEach(inst => {
			inst.popper.destroy();
			inst.elContent.remove();
		});
		service.instances.clear();
	};
	
	// INIT
	document.addEventListener('click', service.onClickAnywhere);

	return service;
}]);
