lm.factory('serviceFileSelect', [ '$rootScope', '$http', 'serviceNotification', 'serviceLimboMedia', function($rootScope, $http, serviceNotification, serviceLimboMedia) {
	var service = new Object();
	
	/* attributes */
	service.items = new Map();		// Map with key like 'file-123' or 'folder-321' and value like {item: theFile, error: anError}. Use for fast lookup and ordered by insert.
	
	service.state = {working: false, sum: 0, current: 0, success: 0, error: 0};
	
	/* internal */
	service.makeKey = function(theFileOrFolder) {
		return (theFileOrFolder.type ? "file-" : "folder-") + theFileOrFolder.id; 
	};
	
	service.makeValue = function(theFileOrFolder) {
		var result = {
			item: theFileOrFolder
		};
		return result;
	};
	
	service.clearErrors = function() {
		service.items.forEach((val, key) => {val.error = null;});
	};
	
	service.actionItem = function(theAction, theWrapper, theFolderTarget, callbackChange) {
		var earl = lmBase.url + (theWrapper.item.type == null ? 'f/folder/' : 'f/file/') + theWrapper.item.id;
		
		var prom = null;
		if('copy' == theAction) {
			prom = $http.post(earl + '/copy', {target: theFolderTarget.id});
		}
		else if('move' == theAction) {
			prom = $http.post(earl + '/move', {target: theFolderTarget.id});
		}
		else if('delete' == theAction) {
			prom = $http.delete(earl);
		}
		
		return prom.then(
			resp => {
				service.remove(theWrapper.item);
				callbackChange.onRemove(theWrapper);
			},
			err => {
				serviceLimboMedia.httpErrorToMsg(err, function(msg) {
					theWrapper.error = msg;
					callbackChange.onChange(theWrapper, theWrapper);
				});
				throw err;
			}
		);
	};

	service.actionInner = async function(theAction, folderTarget, callbackChange) {
		service.state.working = true;
		service.clearErrors();
		service.state.sum = service.items.size;
		service.state.success = 0;
		service.state.error = 0;
		service.state.current = 1;
		
		for (const entry of service.items.entries()) {
			try {
				// await lmBase.delayProm();
				await service.actionItem(theAction, entry[1], folderTarget, callbackChange);
				$rootScope.$apply(() => {service.state.success++;});
			}
			catch(err) {
				$rootScope.$apply(() => {service.state.error++;});
			}
			$rootScope.$apply(() => {service.state.current++;});
		};
		
		return new Promise((resolve, reject) => {
			$rootScope.$apply(() => {
				service.state.working = false;
				var notifyParams = {sum: service.state.sum, ok: service.state.success, error: service.state.error};
				var msg = theAction == 'delete' ? 'MSG_DELETE' : (theAction == 'copy' ? 'MSG_COPY' : 'MSG_MOVE');
				if(service.state.error > 0) {
					serviceNotification.error(msg + '_FAIL', notifyParams);
					reject();
				}
				else {
					serviceNotification.ok(msg + '_OK', notifyParams);
					resolve();
				}
			});
		});
	};
	
	
	/* external */
	service.add = function(theFileOrFolder) {
		service.items.set(service.makeKey(theFileOrFolder), service.makeValue(theFileOrFolder));
	};
	service.remove = function(theFileOrFolder) {
		service.items.delete(service.makeKey(theFileOrFolder));
	};
	service.toggle = function(theFileOrFolder) {
		if(service.has(theFileOrFolder)) service.remove(theFileOrFolder);
		else service.add(theFileOrFolder);
	};
	service.has = function(theFileOrFolder) {
		return service.items.has(service.makeKey(theFileOrFolder));
	};
	service.size = function() {
		return service.items.size;
	};
	service.clear = function() {
		service.items.clear();
	};
	
	service.values = function() {
		var result = [];
		service.items.forEach((val,key) => result.push(val.item));
		return result; /* array */
	};

	service.valuesWrapped = function() {
		return service.items.values(); /* iterator */
	};
	
	// theAction = 'copy'|'move'|'delete'
	service.action = function(action, folderTarget, callbackChange) {
		if(service.state.working) return;
		return service.actionInner(action, folderTarget, callbackChange); /* promise with reject on errors */
	};

	/* Lifecycle */
	$rootScope.$on('lmLogin', function() {
		// service.clear();
	});
	
	$rootScope.$on('lmLogout', function() {
		service.clear();
	});

	
	return service;
}]);
