lm.factory('serviceFullscreen', ['$rootScope', '$http', function($rootScope, $http) {
	var service = new Object();
	
	service.state = new Object();
	service.state.fullscreen = false;
	service.state.aspectScreen = screen.width / screen.height;
	service.state.aspectViewport = window.innerWidth / window.innerHeight;
	service.state.aspectEffective = service.state.aspectViewport; // isFullscreen = aspectScreen : aspectViewport
	
	/* Cross-browser helpers */
	service.isFullscreen = function() {
		return (document.fullscreenElement || document.webkitFullscreenElement || document.mozFullScreenElement || document.msFullscreenElement) ? true : false;
	};

	service.toggleFullscreen = function(enable, element) {
		if(enable) {
			if(element.requestFullscreen) element.requestFullscreen();
			else if(element.webkitRequestFullscreen) element.webkitRequestFullscreen();
			else if(element.mozRequestFullScreen) element.mozRequestFullScreen();
			else if(element.msRequestFullscreen) element.msRequestFullscreen();				
		}
		else {
			if (document.exitFullscreen) document.exitFullscreen();
			else if (document.webkitExitFullscreen) document.webkitExitFullscreen();
			else if (document.mozCancelFullScreen) document.mozCancelFullScreen();
			else if (document.msExitFullscreen) document.msExitFullscreen();
		}
	};

	
	/* Reflect fullscreen and aspect changes */
	service.refreshAspects = function() {
		service.state.aspectScreen = screen.width / screen.height;
		service.state.aspectViewport = window.innerWidth / window.innerHeight;
		service.state.aspectEffective = service.state.fullscreen ? service.state.aspectScreen : service.state.aspectViewport;
	};

	service.refreshAspectsNative = function() {
		$rootScope.$apply(function() {
			service.refreshAspects();
		});
	};

	service.refreshFullscreen = function() {
		$rootScope.$apply(function() {
			service.state.fullscreen = service.isFullscreen();
			service.refreshAspects();
		});
	};
	
	window.addEventListener('orientationchange', service.refreshAspectsNative);
	window.addEventListener('resize', service.refreshAspectsNative);

	document.addEventListener("fullscreenchange", service.refreshFullscreen);
	document.addEventListener("webkitfullscreenchange", service.refreshFullscreen);
	document.addEventListener("mozfullscreenchange", service.refreshFullscreen);
	document.addEventListener("MSFullscreenChange", service.refreshFullscreen);
	
	/* Public methods */
	service.onToggleFullscreen = function(fsElement) {
		service.toggleFullscreen(!service.isFullscreen(),fsElement);
	};
	service.on = function(fsElement) {
		if(!service.isFullscreen()) {
			service.toggleFullscreen(true, fsElement);
		}
	};
	service.off = function() {
		if(service.isFullscreen()) {
			service.toggleFullscreen(false);
		}
	};
	
	service.barsOn = function() {
		document.body.classList.remove("lmNoBars");
	};
	
	service.barsOff = function() {
		document.body.classList.add("lmNoBars");
	};

	/* destructor
	$scope.$on('$destroy', function destroy() {
		document.removeEventListener("fullscreenchange", $scope.refreshFullscreen);
		document.removeEventListener("webkitfullscreenchange", $scope.refreshFullscreen);
		document.removeEventListener("mozfullscreenchange", $scope.refreshFullscreen);
		document.removeEventListener("MSFullscreenChange", $scope.refreshFullscreen);
		window.removeEventListener("orientationchange", $scope.refreshScreenAspect);
	});
	 */
	
	return service;
}]);
