lm.factory('serviceImager', ['$rootScope', '$rootRouter', '$http', 'serviceLimboMedia', 'serviceMediaPlayer', 'serviceFullscreen', 'serviceModal', function($rootScope, $rootRouter, $http, serviceLimboMedia, serviceMediaPlayer, serviceFullscreen, serviceModal) {
	var service = {
		state: {
			file: null,		// current file
			files: [],		// all files
			idx: 0,			// index of current file in all files
			writable: false,// writable means -> allow delete/edit ops. Usually equals folder permission x.write.
			callback: null  // callback to notify onRemove and onChange. 
		}
	};

	service.setImage = function(theImage) {
		service.state.file = theImage;
		service.state.idx = service.state.files ? service.state.files.indexOf(theImage) : -1;
	};
	
	service.openImages = function(theImage, theList, writable, callback) {
		serviceFullscreen.barsOff();
		service.state.files = theList;
		service.state.writable = writable;
		service.state.callback = callback;
		service.setImage(theImage);
	};
	
	service.next = function() {
		for (i = parseInt(service.state.idx) + 1; i < service.state.files.length; i++) {
			var it = service.state.files[i];
			if('IMAGE' == it.type) {
				service.setImage(it);
				return;
			}
		}
		service.close();
	};

	service.prev = function() {
		for (i = parseInt(service.state.idx) - 1; i >= 0; i--) {
			var it = service.state.files[i];
			if('IMAGE' == it.type) {
				service.setImage(it);
				return;
			}
		}
		service.close();
	};
	
	service.delete = function() {
		$http.delete(lmBase.url + 'f/file/' + service.state.file.id).then(
			function success(resp) {
				if(service.state.callback) {
					service.state.callback.onRemove(service.state.file);
				}

				if(service.state.files.length > 0) {
					for (i = parseInt(service.state.idx); i < service.state.files.length; i++) {
						var it = service.state.files[i];
						if('IMAGE' == it.type) {
							service.setImage(it);
							return;
						}
					}
				}
				service.close();
			},
			serviceLimboMedia.handleHttpErrorNew
		);
	};
	
	service.close = function() {
		if(serviceFullscreen.state.fullscreen) {
			serviceFullscreen.toggleFullscreen(false, null);
		}
		serviceFullscreen.barsOn();
		
		service.state.file = null;
		service.state.files = [];
		service.state.callback = null;
	};
	
	service.keyListener = function(event) {
		if(service.state.file) {
			if('ArrowUp' == event.key || 'ArrowLeft' == event.key) {
				$rootScope.$apply(service.prev);
				event.stopPropagation();	// Stop event propagation upwards in DOM
				event.preventDefault();		// Prevent browser default
			}
			else if('ArrowDown' == event.key || 'ArrowRight' == event.key) {
				$rootScope.$apply(service.next);
				event.stopPropagation();	// Stop event propagation upwards in DOM
				event.preventDefault();		// Prevent browser default
			}
			else if('Escape' == event.key) {
				$rootScope.$apply(service.close);
				event.stopPropagation();	// Stop event propagation upwards in DOM
				event.preventDefault();		// Prevent browser default
			}
			else if('Delete' == event.key) {
				$rootScope.$apply(service.delete);
				event.stopPropagation();	// Stop event propagation upwards in DOM
				event.preventDefault();		// Prevent browser default
			}
		}
	};
	window.addEventListener("keydown", service.keyListener);
	
	return service;
}]);
