lm.factory('serviceLimboMedia', [ '$q', '$http', '$location', '$rootScope', 'serviceNotification' , '$translate', '$rootRouter', '$interval', function($q, $http, $location, $rootScope, serviceNotification, $translate, $rootRouter, $interval) {
	var service = new Object();
	
	/* consts */
	service.remoteSettingsName = 'webgui';
	service.initPages = [];
	service.initPages['HOME'] = 'Home';
	service.initPages['FILEMEDIA'] = 'Files';
	service.initPages['BOOKMARKS'] = 'Bookmarks';
	service.initPages['TODOS'] = 'Todos';
	service.initPages['NOTES'] = 'Notes';
	service.initPages['IOT'] = 'IoT';
	
	/* attributes */
	service.userdetails = {
		user: null,
		settingsRemote: null	/* remote storage settings */
	};

	service.settingsLocal = {};	/* local storage settings  */

	/* Promise answering isAuthenticated requests. Possible states: */
	/* null							--> on initialization and after logout */
	/* "return-true-promise"		--> after successfull login */
	/* "get-userdetails-promise"	--> no login but page requested auth */
	service.authpromise = null;

	try { service.haslocalstorage = 'localStorage' in window && window['localStorage'] !== null; }
	catch (e) { service.haslocalstorage = false; }

	service.getUserdetails = function() {
		return service.userdetails;
	}
	
	service.resetUser = function() {
		service.authpromise = null;
		service.userdetails.user = null;
		service.userdetails.settingsRemote = null;
		$translate.use('en');
		$rootScope.$emit('lmLogout');
	};

	service.setUser = function(userData) {
		service.resetUser();
		service.userdetails.user = userData;
	
		$translate.use(userData.language);
		
		try {
			service.userdetails.settingsRemote = angular.fromJson(userData.settings);
		} catch(e) {
			console.log("Error parsing user settings: " + e);
		}
		if(service.userdetails.settingsRemote == null || service.userdetails.settingsRemote == '') {
			service.userdetails.settingsRemote = {};
		}
		
		/* when user is set correctly, we let the authpromise always return true */
		service.authpromise = $q.resolve(true);
		
		$rootScope.$emit('lmLogin');
	};
	
	service.changeLang = function(lang) {
		$translate.use(lang);
		service.userdetails.user.language = lang;
	};
	
	service.handleHttpErrorNew = function(response) {
		var data = response.data;
		var status = response.status;
		console.log("Error Status: " + status + ", Data: " + data);
		if(401 == status) {
			service.resetUser();
			$rootRouter.navigate(['Login']);
		}
		else {
			service.httpErrorToMsg(response, serviceNotification.errorPlain);
		}
		throw response;
	};

	service.httpErrorToMsg = function(response, callbackWithStringArg) {
		var data = response.data;
		var status = response.status;
		
		if(-1 == status) {
			service.httpErrorTranslate({key: 'CONNECTION_ERROR'},callbackWithStringArg);
		}
		else if(data && data.key) {
			service.httpErrorTranslate(data,callbackWithStringArg);
		}
		else {
			callbackWithStringArg("Server Error: " + status);
		}
	};
	
	service.httpErrorTranslate = function(data, callbackWithStringArg) {
		$translate(['ERROR', data.key]).then(function(trans) {
			var msg = trans[data.key];
			
			if(msg == data.key) {
				// no translation available
				msg = data.msg;
			}
			else {
				for(x in data.params) {
					msg = msg.replace("{" + x + "}", data.params[x]);
				}
			}
			
			callbackWithStringArg(msg);
		});
	};
	

	service.hasRole = function(valueRole) {
		if(service.userdetails.user == null) return false;
		return service.userdetails.user.authorities.indexOf(valueRole) >= 0;
	}
	
	
	service.isAuthenticated = function() {
		if(!service.authpromise) {
			service.authpromise = $q(function(resolve, reject) {
				$http.get(lmBase.url + 'user/user?settingKey=' + service.remoteSettingsName).then(
						function success(resp) {
							service.setUser(resp.data);
							resolve(true);
						},
						function error() {
							$rootRouter.navigate(['Login']);
							resolve(false);
						}
				);
			});
		}
		return service.authpromise;
	}
	
	/* Get remote settings of current user. Requires isAuthenticated() to guarantee contents. */
	// service.getRemoteSettings = function() {
	//	return service.userdetails.settingsRemote;
	// }
	service.getRemoteSettings = function(key) {
		if(key) {
			if(service.userdetails.settingsRemote[key] === undefined) {
				service.userdetails.settingsRemote[key] = {};
			}
			return service.userdetails.settingsRemote[key];
		}
		else {
			return service.userdetails.settingsRemote;
		}
	}
	
	
	service.saveRemoteSettings = function() {
		var jsonToSave =  angular.toJson(service.userdetails.settingsRemote);
		$http.post(lmBase.url + 'setting/setting/' + service.remoteSettingsName, jsonToSave, httpCfgText).then(
				function success(resp) { },
				function error(resp) {
					if(403 != resp.status) service.handleHttpErrorNew(resp);	
				}
		);
	};
	
	service.getLocalSettings = function(key) {
		if(service.settingsLocal[key] == null) {
			if(service.haslocalstorage) {
				var data = localStorage.getItem(key);
				service.settingsLocal[key] = (data == null || data == '') ? {} : angular.fromJson(data);  
			}
			else service.settingsLocal[key] = {};
		}
		return service.settingsLocal[key];
	};

	service.saveLocalSettings = function(key) {
		if(service.haslocalstorage) {
			var objectToSave = service.settingsLocal[key];
			var jsonToSave =  angular.toJson(objectToSave);
			localStorage.setItem(key, jsonToSave);
		}
	};
	
	/* external login logout functions */
	service.logout = function() {
		$http.post(lmBase.url + 'user/logout').then(
			function success(resp) {
				service.resetUser();
				$rootRouter.navigate(['Login']);
			},
			service.handleHttpErrorNew
		);
	};
	
	service.login = function(valueUser, valuePass) {
		if(valueUser == null || valuePass == null || valueUser.length == 0 || valuePass.length == 0) {
			serviceNotification.error('MSG_USER_PASS');
			return;
		}
		
		var req = {username: valueUser, password: valuePass, settingKey: service.remoteSettingsName};

		$http.post(lmBase.url + 'user/login',req).then(
				function success(resp) {
					// get user details on success
					$http.get(lmBase.url + 'user/user?settingKey=' + service.remoteSettingsName).then(
							function success(resp) {
								service.setUser(resp.data);
								$rootRouter.navigate(['Home']);
							},
							service.handleHttpErrorNew
					);
				},
				service.handleHttpErrorNew
		);
	};
	
	/* Check regularly for user details to see if local user is still */
	/* authenticated and unchanged, otherwise react accordingly. Why: */
	/* - Auth-Token may have expired/invalidated                      */
	/* - User may have logged in/out in a different tab.              */
	service.keepalive = function() {
		$http.get(lmBase.url + 'user/user?settingKey=' + service.remoteSettingsName).then(
			function success(resp) {
				if(service.userdetails.user == null) {
					if('LAN' != resp.data.name) {
						console.log("KeepAlive: Authorized but there's no local user. -> Autologin.");
						service.setUser(resp.data);
						$rootRouter.navigate(['Home']);
					}
					// else { console.log("KeepAlive: Authorized via LAN but no local user. -> No action, expecting user about to do a manual login."); }
				}
				else if (service.userdetails.user.id != resp.data.id) {
					console.log("KeepAlive: Authorized but as another user. -> Switching.");
					service.setUser(resp.data);
					$rootRouter.navigate(['Home']);
				}
				// else { console.log("KeepAlive: Authorized as local user -> No action."); }
			},
			function error(resp) {
				if(-1 == resp.status) {
					console.log("KeepAlive: Server not reachable -> No action. Might be a temp connection loss, server restart, ...");
					return; 
				}
				else if(401 == resp.status) {
					if(service.userdetails.user != null) {
						console.log("KeepAlive: Unauthorized but there's a local user -> Autologout.");
						service.resetUser();
						$rootRouter.navigate(['Login']);
					}
					// else { console.log("KeepAlive: Unauthorized and there's no local user -> No action."); }
				}
				// else { console.log("KeepAlive: Code " + resp.status + ". -> No action. Not handled atm."); }
			}
		);
	};

	$interval(service.keepalive, 60000);

	return service;
}]);
