/* ========================== */
/* = SERVICE - Media player = */
/* ========================== */
lm.factory('serviceMediaPlayer', ['$rootScope', '$http', 'serviceNotification', 'serviceLimboMedia', 'serviceFullscreen', 'servicePlayerInternal', 'servicePlayerCast', 'serviceSidebar', function($rootScope, $http, serviceNotification, serviceLimboMedia, serviceFullscreen, servicePlayerInternal, servicePlayerCast, serviceSidebar) { 
	var service = new Object();
	
	/* =========== */
	/* = STATICS = */
	/* =========== */
	service.layoutMap = {'P360': '360p', 'P720': '720p', 'P1080': '1080p', 'P2160': '4K'};

	/* ============ */
	/* = SETTINGS = */
	/* ============ */
	service.settingsLocal = {};
	
	/* ========== */
	/* = OUTPUT = */
	/* ========== */
	service.player = servicePlayerInternal;
	
	/* ========= */
	/* = STATE = */
	/* ========= */	
	service.state = {
		file: null,			// current file
		playing: false,
		offset: 0,			// transcoding offset position (seconds)
		position: 0,		// aggregated playback position (offset + position in stream) (seconds)
		volume: 0.5,		// volume (range 0 - 1)
		muted: false,
		format: null,
		formats: [],
		astream: null,
		astreams: [],
		sstream: null,
		sstreams: [],
	};
	
	service.resetState = function(hardreset) {
		service.player.ctrlStop();

		service.state.file = null;
		service.state.playing = false;
		service.state.offset = 0;
		service.state.position = 0;
		service.state.format = null;
		service.state.formats = [];
		service.state.astream = null;
		service.state.astreams = [];
		service.state.sstream = null;
		service.state.sstreams = [];
		if(hardreset) {
			service.state.volume = 0.5;
			service.state.muted = false;
		}
	};
	
	/* ========================== */
	/* = PLAYER STATE CALLBACKS = */
	/* ========================== */
	service.playerStateCallback = function(src, evt, payload) {
		// console.log("Event Player State. Source: " + src.name + ", Name: " + evt + ", Payload: ", payload);
		
		if(evt == lmBase.EVENT_PLAYER_STATE.CONNECTED && src == servicePlayerCast && service.player == servicePlayerInternal) {
			service.stop();
			service.player = servicePlayerCast;
		}
		if(evt == lmBase.EVENT_PLAYER_STATE.DISCONNECTED && src == servicePlayerCast) {
			service.player = servicePlayerInternal;
		}
		if(evt == lmBase.EVENT_PLAYER_STATE.POSITION) {
			service.state.position = Number(service.state.offset) + payload;
		}
		if(evt == lmBase.EVENT_PLAYER_STATE.PLAY) {
			service.state.playing = payload;
		}
		if(evt == lmBase.EVENT_PLAYER_STATE.END) {
			if(service.pl.repeat && service.pl.repeatOne) service.intPlayAgain();
			else service.playNext();
		}
	};
	
	/* ============ */
	/* = PLAYLIST = */
	/* ============ */	
	
	service.pl = {
		playlist: {id: 0, name: 'last', items: []},
		idx: 0,
		random: false,
		repeat: false,
		repeatOne: false
	};

	service.resetPlaylist = function() {
		service.pl.playlist = {id: 0, name: 'last', items: []};
		service.pl.idx = 0;
		service.pl.random = false;
		service.pl.repeat = false;
		service.pl.repeatOne = false;
	};


	/* ============ */
	/* = INTERNAL = */
	/* ============ */

	/* Play some media file. Load details (formats, ...) when not available. */
	service.intPlayFile = function(valueFile) {
		// No details required for STREAMs.
		if(valueFile.type == 'STREAM') {
			service.intPlayFileWithDetails(valueFile);
			return;
		}
		
		/* Loading files with details (formats, ...) initially is expensive. Therefore we lazy-load details here as soon as required. */
		if(valueFile.formats === undefined || valueFile.formats.length == 0) {
			$http.get(lmBase.url + 'f/file/' + valueFile.id).then(
				function success(resp) {
					valueFile.formats = resp.data.formats;
					service.intPlayFileWithDetails(valueFile);
				},
				serviceLimboMedia.handleHttpErrorNew
			);
		} else {
			service.intPlayFileWithDetails(valueFile);
		}
	};
	
	/* Play some media file. Call as soon as file details (formats, ...) are available. */
	service.intPlayFileWithDetails = function(valueFile) {
		service.resetState();
		service.state.file = valueFile;
		
		if(service.state.file.type == 'AUDIO') {
			service.playAudio(valueFile);
		}
		else if(service.state.file.type == 'VIDEO') {
			service.playVideo(valueFile);
		}
		else if(service.state.file.type == 'STREAM') {
			service.playStream(valueFile);
		}
	};

	service.playAudio = function(valueFile) {
		var supportedStandard = null;
		for(x in valueFile.formats) {
			var af = valueFile.formats[x];
			if(service.player.supportsAudio(af.standard)) {
				supportedStandard = af.standard;
				break;
			}
		}

		var playurl = lmBase.url + 'f/streamAudio/' + valueFile.id +'/x' + '?standard=' + (supportedStandard == null ? 'WEBM_VORBIS' : supportedStandard);

		// TODO: get mime type somewhere .. for audios backend doesnt deliver ... maybe add
		// service.player.ctrlLoad(service.state.file.id, playurl, service.state.format.mime);
		service.player.ctrlLoad(service.state.file.id, playurl, null);
	}
	
	service.playVideo = function(valueFile) {
		/* extract audio and sub streams */
		for(x in valueFile.formats) {
			var format = valueFile.formats[x];
			if(format.original) {
				for(y in format.streams) {
					var stream = format.streams[y];
					if("AUDIO" == stream.type) {
						service.state.astreams.push(stream);
					}
					if("SUB" == stream.type && ('SUBRIP' == stream.codec || 'WEBVTT' == stream.codec)) {
						service.state.sstreams.push(stream);
					}
				}
				if(service.state.astreams.length > 0) {
					service.state.astream = service.state.astreams[0];
				}
			}
		}
		
		
		/* Add available streams, original and supported standards */
		for(x in valueFile.formats) {
			var format = valueFile.formats[x];
			if(format.original || service.player.supportsVideo(format.standard)) {
				service.state.formats.push(format);
			}
		}

		/* Add transcoding streams, only supported standards */
		for(tfi in lmBase.videoFormats) {
			var tf = lmBase.videoFormats[tfi];
			var alreadyThere = false;
			for(x in valueFile.formats) {
				var format = valueFile.formats[x];
				if(!format.original && tf.standard == format.standard && tf.layout == format.layout) {
					alreadyThere = true;
					break;
				}
			}
			if(!alreadyThere && service.player.supportsVideo(tf.standard)) {
				service.state.formats.push(tf);
			}
		}
		
		/* sort */
		service.state.formats.sort(lmBase.compareVideoFormat);
		
		/* add guinames and metainfos */
		for(x in service.state.formats) {
			var format = service.state.formats[x];
			format.guinamesmall = format.original ? 'orig.' : service.layoutMap[format.layout];
			format.guiname = (format.layout == null ? 'Custom' : service.layoutMap[format.layout]);
			format.guiname += ' ' + format.mime.substring(6);

			if(format.original) format.guiname += ' (original)';
			else if(format.transcoding) format.guiname += ' (transcode)';
			
			/* add nicesize and urls, and playable */
			format.url = lmBase.url + 'f/streamVideo/' + valueFile.id + '/' + valueFile.urlname;
			if(!format.original) {
				format.url += '.' + format.standard + '?standard=' + format.standard + '&layout=' + format.layout + '&sau=666';
			}
		}
		
		/* Select format to play. Prefer last selected of ['P360','P720','P1080']. */
		if('P360' == service.settingsLocal.videoLayout || 'P720' == service.settingsLocal.videoLayout || 'P1080' == service.settingsLocal.videoLayout) {
			var possibleFormats = service.state.formats.filter(function(valFormat) {
				return service.settingsLocal.videoLayout == valFormat.layout;
			});

			if(possibleFormats.length > 0) {
				service.state.format = possibleFormats[0];
			}
		}
		else if(service.state.formats.length > 0) {
			service.state.format = service.state.formats[0];
		}

		service.refreshVideo(0);
	}
	
	service.playStream = function(valueFile) {
		/* extract audio and sub streams */
		// not yet
		
		/* Add available streams, original and supported standards */
		// no original yet - maybe create some kind of pass-through

		/* Add transcoding streams, only supported standards */
		for(tfi in lmBase.videoFormats) {
			var tf = lmBase.videoFormats[tfi];
			if(service.player.supportsVideo(tf.standard)) {
				service.state.formats.push(tf);
			}
		}
		
		/* Add fake original for passthrough streaming */
		var formatOriginal = {
			original: true,
			layout: null,
			mime: null,
			size: 0,
			standard: null,
			streams: [],
			transcoding: false
		};
		service.state.formats.push(formatOriginal);

		/* sort */
		service.state.formats.sort(lmBase.compareVideoFormat);
		
		/* add guinames and metainfos */
		for(x in service.state.formats) {
			var format = service.state.formats[x];
			format.guinamesmall = format.original ? 'orig.' : service.layoutMap[format.layout];
			format.guiname = (format.layout == null ? 'Custom' : service.layoutMap[format.layout]);
			format.guiname += format.mime ? (' ' + format.mime.substring(6)) : '';

			if(format.original) format.guiname += ' (original)';
			else if(format.transcoding) format.guiname += ' (transcode)';
			
			/* add nicesize and urls, and playable */
			format.url = lmBase.url + 'f/streamVideo/' + valueFile.id + '/' + valueFile.urlname;
			if(!format.original) {
				format.url += '.' + format.standard + '?standard=' + format.standard + '&layout=' + format.layout + '&sau=666';
			}
		}
		
		/* Select format to play. Prefer last selected of ['P360','P720','P1080']. */
		if('P360' == service.settingsLocal.videoLayout || 'P720' == service.settingsLocal.videoLayout || 'P1080' == service.settingsLocal.videoLayout) {
			var possibleFormats = service.state.formats.filter(function(valFormat) {
				return service.settingsLocal.videoLayout == valFormat.layout;
			});

			if(possibleFormats.length > 0) {
				service.state.format = possibleFormats[0];
			}
		}
		else if(service.state.formats.length > 0) {
			service.state.format = service.state.formats[0];
		}

		service.refreshVideo(0);
	}
	
	/*
	 * changePosition:
	 *  -1 = Keep position
	 *   0 = Reset to start
	 * 1-x = Set this position
	 * not working right now: switching from transcoding to non-transcoding
	 */
	service.refreshVideo = function(changePosition) {
		if(changePosition == -1) {
			service.state.offset = service.state.format.transcoding ? Math.floor(service.state.position) : 0; 
		}
		else {
			service.state.offset = changePosition;
		}

		var playurl = lmBase.url + 'f/streamVideo/' + service.state.file.id + '/x';

		if(!service.state.format.original) {
			playurl += '?standard=' + service.state.format.standard + '&layout=' + service.state.format.layout;
		}
		
		if(service.state.file.type != 'STREAM') {
			if(service.state.format.transcoding) {
				if(service.state.offset != 0) {
					playurl += "&ss=" + service.state.offset;
				}
				if(service.state.astream != null) {
					playurl += "&sau=" + service.state.astream.idx;
				}
			}
		}

		var thetracks = [];
		service.state.sstreams.forEach(stream => {
			if(stream.idx != -1) {
				var x = {
					stream: stream,
					earl: lmBase.url + 'f/streamVideoSub/' + service.state.file.id + '/' + stream.idx + (service.state.offset == 0 ? '' : ('?ss=' + service.state.offset))
				};
				thetracks.push(x);
			}
		});
		
		service.player.ctrlLoad(service.state.file.id, playurl, service.state.format.mime, thetracks, service.state.sstream ? service.state.sstream.idx : null);
	};

	/* ===================== */
	/* = INTERNAL PLAYLIST = */
	/* ===================== */
	
	/* Play item at playlist position. */
	service.intPlayIndex = function(valueIndex) {
		if(service.pl.playlist.items.length == 0 || valueIndex < 0) {
			return;
		}

		if(valueIndex >= service.pl.playlist.items.length) {
			if(service.pl.repeat && !service.pl.repeatOne) valueIndex = 0;
			else return;
		}

		service.pl.idx = valueIndex;

		var fileToPlay = service.pl.playlist.items[valueIndex];
		service.intPlayFile(fileToPlay);
	};
	
	service.intPlayRandom = function() {
		service.intPlayIndex(Math.floor(Math.random() * service.pl.playlist.items.length));
	};
	
	service.intPlayAgain = function() {
		service.intPlayIndex(service.pl.idx);
	}
	
	
	/* ============================ */
	/* = EXTERNAL COMMANDS PLAYER = */
	/* ============================ */	
	
	/* Set volume. Input parameter range: number between 0.00 - 1.00. */
	service.changeVolume = function(valueNewVolume) {
		service.state.volume = valueNewVolume;
		service.player.ctrlVolume(valueNewVolume);
	};
	
	service.toggleVolume = function() {
		service.state.muted = !service.state.muted;
		service.player.ctrlMute(service.state.muted);
	};
	
	/* Set position. Input parameter range: number between 0.00 - 1.00. */
	service.changePosition = function(valueNewPosition) {
		if(!service.state.file) return;

		var newpos = valueNewPosition * service.state.file.duration;
		
		if('VIDEO' == service.state.file.type && service.state.format.transcoding) {
			service.refreshVideo(Math.round(newpos));
		}
		else {
			service.player.ctrlSeek(newpos);
		}
	}
	
	/* Play if paused, pause if playing. */
	service.playPause = function() {
		if(service.state.file == null) {
			service.playCurrent();
		}
		else {
			service.player.ctrlPlayPause();
		}
	};	
	
	/* Stop player and reset state. */
	service.stop = function() {
		service.resetState();
		serviceFullscreen.off();
		service.player.ctrlStop();
	};
	
	/* Reset current track. */
	service.reset = function() {
		if(service.state.file == null || !service.binding.bound) return;
		service.intPlayFile(service.state.file);
	};
	
	/* Change video stream. */
	service.changeFormat = function(format) {
		service.state.format = format;
		service.refreshVideo(-1);
		if(format.layout) {
			service.settingsLocal.videoLayout = format.layout;
			service.settingsLocalSave();
		}
	};
	
	/* Change audio stream. */
	service.changeAudioStream = function(as) {
		service.state.astream = as;
		service.refreshVideo(-1);
	};
	
	/* Change subtitle stream. */
	service.changeSubStream = function(ss) {
		service.state.sstream = ss;
		service.player.ctrlSubtitle(ss == null ? null : ss.idx);
	};
	
	service.toggleFullscreen = function() {
		servicePlayerInternal.ctrlFullscreenToggle();
	};

	
	/* ============================== */
	/* = EXTERNAL COMMANDS PLAYLIST = */
	/* ============================== */
	
	service.playIndex = function(valIndex) {
		service.intPlayIndex(valIndex);
	}
	
	service.playPrev = function() {
		if(service.pl.random) service.intPlayRandom();
		else service.intPlayIndex(service.pl.idx - 1);
	};
	
	service.playNext = function() {
		if(service.pl.random) service.intPlayRandom();
		else service.intPlayIndex(service.pl.idx + 1);
	};
	
	service.playCurrent = function() {
		if(service.pl.playlist.items.length > 0) {
			if(service.pl.idx < service.pl.playlist.items.length) {
				service.intPlayIndex(service.pl.idx);
			}
			else if(service.pl.random) {
				service.intPlayRandom();
			}
			else {
				service.intPlayIndex(0);
			}
		}
	}
	
	service.playFile = function(valueFile) {
		service.pl.playlist.items = [valueFile];
		service.intPlayIndex(0);
		service.playlistOnChange();
		serviceSidebar.setShow(true);
	};

	service.playFiles = function(valueFiles) {
		service.pl.playlist.items = new Array();
		for(x in valueFiles) {
			service.pl.playlist.items.push(valueFiles[x]);
		}
		service.intPlayIndex(0);
		service.playlistOnChange();
		serviceSidebar.setShow(true);
	};

	service.addFile = function(valueFile) {
		service.pl.playlist.items.push(valueFile);
		service.playlistOnChange();
	};
	
	service.addFiles = function(valueFiles) {
		for(x in valueFiles) {
			service.pl.playlist.items.push(valueFiles[x]);
		}
		service.playlistOnChange();
	};

	service.addPlayFile = function(valueFile) {
		service.pl.playlist.items.push(valueFile);
		service.intPlayIndex(service.pl.playlist.items.length - 1);
		service.playlistOnChange();
		serviceSidebar.setShow(true);
	};
	
	/* playlist stuff */
	service.playlistsLoadLast = function() {
		$http.get(lmBase.url + 'f/playlists/list').then(
			function success(resp) {
				var found = false;
				for(y in resp.data) {
					var pl = resp.data[y];
					if('last' == pl.name) {
						service.playlistLoad(pl,false);
						found = true;
					}
				}
				if(!found) {
					service.pl.playlist = {id: 0, name: 'last', items: []};
				}
			},
			serviceLimboMedia.handleHttpErrorNew
		);
	};
	
	service.playlistLoad = function(playlist,playit) {
		$http.get(lmBase.url + 'f/playlist/' + playlist.id).then(
			function success(resp) {
				for(x in resp.data.items) {
					lmBase.pimpLmFile(resp.data.items[x]);
				}
				service.pl.playlist = resp.data;
				if(playit) service.intPlayIndex(0);
			},
			serviceLimboMedia.handleHttpErrorNew
		);
	};	

	service.playlistSave = function(valName) {
		var req = {name: valName, items: lmBase.arrayToIdArray(service.pl.playlist.items)};
		$http.post(lmBase.url + 'f/playlist',req).then(
			function success(resp) {
				service.pl.playlist.id = resp.data.id;
				service.pl.playlist.name = resp.data.name;
				serviceNotification.ok('MSG_SAVED');
			},
			serviceLimboMedia.handleHttpErrorNew
		);
	};

	service.playlistSaveLast = function() {
		var req = {name: 'last', items: lmBase.arrayToIdArray(service.pl.playlist.items)};
		$http.post(lmBase.url + 'f/playlist',req).then(
			function success(resp) {},
			serviceLimboMedia.handleHttpErrorNew
		);
	};
	
	service.playlistDelete = function(playlist) {
		$http.delete(lmBase.url + 'f/playlist/' + playlist.id).then(
			function success(resp) {
				service.playlistsLoadLast();
			},
			serviceLimboMedia.handleHttpErrorNew
		);
	};
	
	service.playlistClear = function() {
		service.pl.playlist.items = [];
		service.playlistOnChange();
	};	
	
	service.playlistOnChange = function() {
		if(serviceLimboMedia.hasRole('ROLE_WRITE')) {
			service.playlistSaveLast();
		}
	};

	$rootScope.$on('lmLogin', function() {
		service.playlistsLoadLast();
	});
	
	$rootScope.$on('lmLogout', function() {
		service.resetState(true);
		service.resetPlaylist();
	});
	
	service.settingsLocalLoad = function() {
		service.settingsLocal = serviceLimboMedia.getLocalSettings('player');
		if(service.settingsLocal.videoLayout == null) service.settingsLocal.videoLayout = 'P720';
	};
	
	service.settingsLocalSave = function() {
		serviceLimboMedia.saveLocalSettings('player');
	};

	service.settingsLocalLoad();
	
	servicePlayerInternal.listenerSet(service.playerStateCallback);
	servicePlayerCast.listenerSet(service.playerStateCallback);

	return service;
}]);
