lm.factory('serviceModal', [ '$q', '$rootScope', 'serviceFullscreen', function($q, $rootScope, serviceFullscreen) {
	var service = new Object();
	
	/* Stack full of modal instances */
	service.stack = [];
	
	service.internalClose = function(valueInstance) {
		var idx = service.stack.indexOf(valueInstance);
		if(idx == -1) {
			console.log("Cannot close modal. Not found in stack.");
			return null;
		}
		var instance = service.stack[idx];
		service.stack.splice(idx,1);
		serviceFullscreen.barsOn();
		return instance;
	};
	
	/* Callback for modal component on dismiss/close without action */
	service.dismiss = function(valueInstance) {
		if(valueInstance.state.closable) {
			var instance = service.internalClose(valueInstance);
			if(instance) {
				instance.defProm.reject(valueInstance.state.closeResult);
			}
		}
	};
	
	/* Callback for modal component on success action with optional result */
	service.done = function(valueInstance, valueResult) {
		if(valueInstance.state.closable) {
			var instance = service.internalClose(valueInstance);
			if(instance) {
				instance.defProm.resolve(valueResult);
			}
		}
	};
	
	/* Open modal (params: passed straight to dialog, options: modal wrapper options) */
	service.show = function(valueName, valueParams, valueOptions) {
		var valueOptions = valueOptions ? valueOptions : {
			promiseClose: null,		// If given, user (and inner) cannot close modal but this promise does. For "loading/working/wait" blocking modals.
			maximize: false,		// Open maximized. Note that modal should support "growing".
			backdropClose: false	// Close on backdrop click.
		};
		var instance = {
			name: valueName,
			options: valueOptions,
			state: { /* State is used for wrapper vs. inner communication. */
				closable: !!!valueOptions.promiseClose,	/* inner can change this one to prevent closing by wrapper (X or EscKey) */
				closeResult: false						/* inner can change this to return on reject. Feel free to change type */
			},
			params: valueParams ? valueParams : {},	// make sure it's never undefined.
			defProm: $q.defer()
		};

		serviceFullscreen.barsOff();
		service.stack.push(instance);
		
		if(valueOptions.promiseClose) {
			valueOptions.promiseClose.then(
				function(result) {
					instance.state.closable = true;
					service.dismiss(instance);
				},
				function(error) {
					instance.state.closable = true;
					service.dismiss(instance);
				}
			);
		}

		return instance.defProm.promise;
	};
	
	service.keyListener = function(event) {
		if (event.defaultPrevented) return; // Do nothing if event already handled

		if(service.stack.length > 0 && 'Escape' == event.key) {
			var latestInstance = service.stack[service.stack.length - 1];
			if(latestInstance.state.closable) {
				$rootScope.$apply(() => {
					service.dismiss(latestInstance);
				});
				event.preventDefault(); // Consume the event so it doesn't get handled twice
			}
		}
	};
	window.addEventListener("keydown", service.keyListener);

	return service;
}]);
