/* ========================= */
/* = SERVICE - Cast sender = */
/* ========================= */
lm.factory('servicePlayerCast', ['$rootScope', '$http', 'serviceNotification', 'serviceLimboMedia', function($rootScope, $http, serviceNotification, serviceLimboMedia) { 
	var service = new Object();
	service.name = 'cast';
	
	/* ========= */
	/* = STATE = */
	/* ========= */
	service.castLoaded = false;		// SDK loaded and API initialized and read?
	service.castSession = null;
	service.castContext = null;
	service.castPlayer = null;
	service.castControl = null;
	service.castMedia = null;		// Media loaded currently. Required to change tracks while playing.

	/* ======== */
	/* = INFO = */
	/* ======== */
	service.supportsVideo = function(valStandard) {
		return valStandard == 'WEBM' || valStandard == 'MP4';
	};
	service.supportsAudio = function(valStandard) {
		return valStandard == 'MP3'  || valStandard == 'OGG_VORBIS' || valStandard == 'WEBM_VORBIS' ||
			   valStandard == 'FLAC' || valStandard == 'WAV'        || valStandard == 'M4A' ||
			   valStandard == 'AAC'  || valStandard == 'OPUS';  
	};

	/* ================== */
	/* = STATE LISTENER = */
	/* ================== */
	service.listener = null;
	
	service.listenerSet = function(valListener) {
		service.listener = valListener;
		service.init();
	};
	
	service.listenerNotify = function(evt, payload) {
		if(!service.listener) return;
		$rootScope.$apply(function() {
			service.listener(service, evt, payload);
		});
	};


	/* =========== */
	/* = CONTROL = */
	/* =========== */
	service.ctrlLoad = function(fileId, url, contentType, tracks, trackActiveIdx) {
		$http.get(lmBase.url + 'f/file/' + fileId + '/token').then(
			function success(resp) {
				// We need the full absolute url
				url = lmBase.urlFull + url;
				
				// And we need the token
				url = lmBase.urlAddParam(url, 'token', resp.data.id);
				
				// Add subtitle tracks.
				var subtracks = [];
				if(tracks) {
					tracks.forEach(track => {
						var subtrack = new chrome.cast.media.Track(track.stream.idx, chrome.cast.media.TrackType.TEXT);
						subtrack.subtype = chrome.cast.media.TextTrackType.SUBTITLES;
						subtrack.trackContentType = 'text/vtt';
						subtrack.trackContentId = lmBase.urlFull + lmBase.urlAddParam(lmBase.urlAddParam(track.earl, 'codec', 'WEBVTT'), 'token', resp.data.id);
						subtrack.name = track.stream.info;
						subtrack.language = 'en-US'; // we have strange 3 letter lang codes from ffmpeg ... too lazy to convert.
						subtrack.customData = null;
						subtracks.push(subtrack);
					});
				}
				
				// Now let cast play that thang
				var ses = cast.framework.CastContext.getInstance().getCurrentSession();
				var mediaInfo = new chrome.cast.media.MediaInfo(url);
				mediaInfo.contentType = contentType;
				mediaInfo.textTrackStyle = new chrome.cast.media.TextTrackStyle();
				mediaInfo.tracks = subtracks;
				var request = new chrome.cast.media.LoadRequest(mediaInfo);
				request.activeTrackIds = trackActiveIdx == null ? [] : [trackActiveIdx];
				
				// Guides says there's a result as promise, Reference says result in passed callback functions but none gets called in reality.
				ses.loadMedia(request);
			},
			serviceLimboMedia.handleHttpErrorNew
		);
	};
	service.ctrlVolume = function(vol) {
		service.castPlayer.volumeLevel = vol;
		service.castControl.setVolumeLevel();
	};
	service.ctrlMute = function(mute) {
		if(mute && !service.castPlayer.isMuted || !mute && service.castPlayer.isMuted) {
			service.castControl.muteOrUnmute();	
		}
	};
	service.ctrlPlayPause = function() {
		service.castControl.playOrPause();
	};
	service.ctrlStop = function() {
		service.castControl.stop();
	};
	service.ctrlSeek = function(pos) {
		service.castPlayer.currentTime = pos;
		service.castControl.seek();
	};
	service.ctrlFullscreenToggle = function() {
		// No support for FS toggle, so do nothing.
	}
	service.ctrlSubtitle = function(idx) {
		var trackIds = idx == null ? [] : [idx];
		var ses = cast.framework.CastContext.getInstance().getCurrentSession();
		var media = ses.getMediaSession();
		var request = new chrome.cast.media.EditTracksInfoRequest(trackIds);
		media.editTracksInfo(request);
	}
	
	
	/* ======== */
	/* = INIT = */
	/* ======== */
	service.init = function() {
		// Make sure sdk is loaded AND listener is registered, so we dont miss events on startup.
		if(!service.castLoaded || !service.listener) return;

		cast.framework.CastContext.getInstance().setOptions({receiverApplicationId: chrome.cast.media.DEFAULT_MEDIA_RECEIVER_APP_ID});
		
		// Listen to connect/disconnect (upon using cast icon)
		service.castContext = cast.framework.CastContext.getInstance();
		service.castContext.addEventListener(cast.framework.CastContextEventType.SESSION_STATE_CHANGED, function(event) {
			switch (event.sessionState) {
				case cast.framework.SessionState.SESSION_STARTED:
				case cast.framework.SessionState.SESSION_RESUMED:
					service.listenerNotify(lmBase.EVENT_PLAYER_STATE.CONNECTED, null);
					break;
				case cast.framework.SessionState.SESSION_ENDED:
					service.listenerNotify(lmBase.EVENT_PLAYER_STATE.DISCONNECTED, null);
        			break;
    		}
		});
		
 		// Listen to player state events
		service.castPlayer = new cast.framework.RemotePlayer();
		service.castControl = new cast.framework.RemotePlayerController(service.castPlayer);
		service.castControl.addEventListener(cast.framework.RemotePlayerEventType.ANY_CHANGE, function(event) {
			if(event.field == 'currentTime') {
				service.listenerNotify(lmBase.EVENT_PLAYER_STATE.POSITION, event.value);
			}
			else if(event.field == 'playerState') {
				service.listenerNotify(lmBase.EVENT_PLAYER_STATE.PLAY, event.value == 'PLAYING' || event.value == 'BUFFERING'); // Other states: 'IDLE'
			}
			else {
				// console.log("EVENT PLAYER:", event);
			}
		});
		
	};
	
	window['__onGCastApiAvailable'] = function(isAvailable) {
  		if (isAvailable) {
			service.castLoaded = true;
			$rootScope.$apply(function() {
				service.init();
			});
  		}
	};
	
	return service;
}]);
