/* ========================= */
/* = SERVICE - Cast sender = */
/* ========================= */
lm.factory('servicePlayerInternal', ['$rootScope', '$http', 'serviceNotification', 'serviceLimboMedia', 'serviceFullscreen', function($rootScope, $http, serviceNotification, serviceLimboMedia, serviceFullscreen) { 
	var service = new Object();
	service.name = 'internal';
	
	service.elementVideo = null;	// Bound video element
	service.elementFrame = null;	// Bound frame for fullscreening
	
	service.supportedVideo = [];	// effectively static: filled on first binding
	service.supportedAudio = [];	// effectively static: filled on first binding
	
	service.activeTrackIdx = null;	// null means no sub and is used because idx might be 0 as well.
	
	/* ======== */
	/* = INFO = */
	/* ======== */
	service.supportsVideo = function(valStandard) {
		return service.supportedVideo.indexOf(valStandard) != -1;
	};
	service.supportsAudio = function(valStandard) {
		return service.supportedAudio.indexOf(valStandard) != -1;
	};
	
	/* ================== */
	/* = STATE LISTENER = */
	/* ================== */
	service.listener = null;
	
	service.listenerSet = function(valListener) {
		service.listener = valListener;
	};
	
	service.listenerNotify = function(evt, payload) {
		if(!service.listener) return;
		$rootScope.$apply(function() {
			service.listener(service, evt, payload);
		});
	};

	/* =========== */
	/* = CONTROL = */
	/* =========== */
	service.ctrlLoad = function(fileId, url, contentType, tracks, trackActiveIdx) {
		if(!service.elementVideo) return;
		
		service.elementVideo.pause();
		try { service.elementVideo.currentTime = 0; } catch(error) {};
		service.activeTrackIdx = trackActiveIdx;
		
		// Clear subtitle tracks. We can use lastChild because there shouldn't be other childern.
		while (service.elementVideo.lastChild) {
			service.elementVideo.removeChild(service.elementVideo.lastChild);
  		}
		
		// Add subtitle tracks via classic DOM shitz because TextTracks API doesn't allow later remove.
		if(tracks) {
			tracks.forEach((track, i) => {
				var t = document.createElement('track');
				t.id = 'idx-' + track.stream.idx;
				t.label = track.stream.info;
				t.srclang = track.stream.language;
				t.kind = 'subtitles';
				t.src = lmBase.urlAddParam(track.earl, 'codec', 'WEBVTT');
				// t.addEventListener("load", function() { console.log("Track loaded:", this); });
				service.elementVideo.appendChild(t);
			});
		}
		
		// Play that thang		
		service.elementVideo.src = url;
		service.elementVideo.play();
	}
	service.ctrlVolume = function(vol /* Range: 0.00 - 1.00 */) {
		service.elementVideo.volume = vol;
	}
	service.ctrlMute = function(mute) {
		service.elementVideo.muted = mute;
	}
	service.ctrlPlayPause = function() {
		if (service.elementVideo.paused || service.elementVideo.ended) service.elementVideo.play();
		else service.elementVideo.pause();
	}
	service.ctrlStop = function() {
		if(!service.elementVideo) return;
		service.elementVideo.pause();
		try { service.elementVideo.currentTime = 0; } catch(error) {};
	}
	service.ctrlSeek = function(pos) {
		service.elementVideo.currentTime = pos;
	}
	service.ctrlFullscreenToggle = function() {
		serviceFullscreen.onToggleFullscreen(service.elementFrame);
	}
	service.ctrlSubtitle = function(idx) {
		service.activeTrackIdx = idx;
		service.internalChangeSub(idx);
	}

	/* ============ */
	/* = INTERNAL = */
	/* ============ */

	// Show the subtitle with given idx (and disable others).
	// Note that timing is very critical here:
	// - If the video is playing already, everything's fine and sub can be changed easily at any time. 
	// - If you want to set sub on startup you need to wait for video elements 'loadedmetadata' event,
	//   which tells that duration, dimensions and text tracks have been processed. Otherwise mode
	//   will be reset or other tracks may activated additionally for whatever reason.
	service.internalChangeSub = function(idx) {
		for (const tt of service.elementVideo.textTracks) {
			tt.mode = (tt.id == 'idx-' + idx) ? "showing" : "disabled";
		};
	}

	/* ================== */
	/* = BINDING = */
	/* ================== */
	service.bind = function(elementVideo, elementFrame) {
		service.elementVideo = elementVideo;
		service.elementFrame = elementFrame;
		
		// Check supported video and audio containers and codecs:
		service.supportedVideo = [];
		service.supportedAudio = [];
		if(service.elementVideo.canPlayType("video/webm").match(/maybe|probably/i)) service.supportedVideo.push("WEBM");
		if(service.elementVideo.canPlayType("video/mp4").match(/maybe|probably/i)) service.supportedVideo.push("MP4");
		if(service.elementVideo.canPlayType("audio/webm").match(/maybe|probably/i)) service.supportedAudio.push("WEBM_VORBIS");
		if(service.elementVideo.canPlayType("audio/mp3").match(/maybe|probably/i)) service.supportedAudio.push("MP3");
		if(service.elementVideo.canPlayType("audio/ogg").match(/maybe|probably/i)) service.supportedAudio.push("OGG_VORBIS");
		// console.log("Supported Formats. Video: " + service.supportedVideo + " --- Audio: " + service.supportedAudio);
		
		// React on state changes
		service.elementVideo.addEventListener('loadedmetadata', function() {
			service.internalChangeSub(service.activeTrackIdx);
		});
		service.elementVideo.addEventListener('timeupdate', function() {
			var src = this; // video or audio element. alt: dingens.srcElement. src == service.elAudioPlayer
			service.listenerNotify(lmBase.EVENT_PLAYER_STATE.POSITION, Number(src.currentTime));
		});
		service.elementVideo.addEventListener('playing', function() {
			service.listenerNotify(lmBase.EVENT_PLAYER_STATE.PLAY, true);
		});
		service.elementVideo.addEventListener('pause', function() {
			service.listenerNotify(lmBase.EVENT_PLAYER_STATE.PLAY, false);
		});
		service.elementVideo.addEventListener('ended', function() {
			service.listenerNotify(lmBase.EVENT_PLAYER_STATE.END, null);
		});

		// Initial volume
		// service.binding.video.volume = service.state.volume;
		// Ready to rumble
	}

	
	return service;
}]);